.\" -*- nroff -*-
.TH simplify 1 "3 Mar 2000" "lpsms 0.5.2"
.\" Turn off hyphenation
.hlm 0
.\" Turn off adjustment
.na
.SH NAME
simplify - perform (stable-model-preserving) conversions on a logic program
.SH SYNOPSIS
.B simplify
.RI "[" options "]"
.RI "[" file "...]"
.\" Back to standard adjustment
.ad
.\" Allow hyphenation, omit if you want no hyphenation in the man page
.hlm 1

.SH DESCRIPTION
The program
.B simplify
takes as input a program in
.B smodels 1
format, and produces as output another program in
.B smodels 1
format.  If the
.B -K
argument is not specified, it  eliminates any positive and negative
requirements that may have been specified in the \fCcompute\fR
statement.  For example, if the \fCcompute\fR statment is:
.RS
\fCcompute { a, b, not c }\fR
.RE
.B simplify
first removes every rule with \fCnot a\fR or \fCnot b\fR in the body.  It then adds the rules:
.RS
\fCGpr00000 :- not a, not Gpr00000.\fR
.br
\fCGpr00000 :- not b, not Gpr00000.\fR
.RE
Then, every rule with \fCc\fR in its body is eliminated, and \fCnot c\fR is removed from the body of
all remaining rules.  Finally, \fCnot c\fR is appended to the body of all rules with head \fCc\fR.
These transformations ensure that the stable models of the simplified program are those stable
models of the original program which contain \fCa\fR, \fCb\fR, and \fCnot c\fR. Note that this
step is necessary for programs which will be run under lderes or sderes\(emthe \fCcompute\fR
statement may be encoded in the output of the grounder even when it did not occur before.

If
.B -K
is specified, the program is still simplified somewhat with respect to
requirements (rules are still eliminated, and negative requirements are
removed from bodies where they appear negatively).  However, no new
rules are added, and no rules are appended to.

If the
.B -u
argument is specified,
.B simplify
also makes killing atoms unique.  A killing rule is a rule of the form:
.RS
\fCf :- a, b, not c, not f.\fR
.RE
In this rule,
.B f
is the local killing atom.  An atom is a killing atom if and only if it appears in the program only
as a local killing atom (possibly in different rules)
.B simplify -u
makes these unique (such that each killing rule has a different killing atom).  For example, the
segment:
.RS
\fCf :- a, b, not f.\fR
.br
\fCf :- c, not d, not f.\fR
.br
\fCf :- not e, not f.\fR
.RE
would become
.RS
\fCGka00000 :- a, b, not Gka00000.\fR
.br
\fCGKa00001 :- c, not d, not Gka00001.\fR
.br
\fCf :- not e, not f.\fR
.RE
All the new atoms generated by
.B simplify -u
have the form \fCGka\fInnnnn\fR; because
.B lparse 
takes symbols beginning with uppercase characters to be variables, there should be no conflict with
the program\(aqs atom names.
.B simplify -u
performs the \fCcompute\fR statement conversion before uniquification, so new killing atoms
introduced by the conversion are also made unique

Note that
.B simplify
does not currently perform any actual simplifications, other than converting logic programs extended
with the
.B smodels
compute statement into pure logic programs.  In fact, with the
.B -u
argument,
.B simplify
enlarges the input program (if there exist multiple killing rules).  The new program, however,
usually has a better stratification than the original.

Finally,
.B simplify
performs a few conversions of atom names.  First, the atom
.BR &false ,
generated by earlier versions of lparse to represent global constraints,
is renamed to 
.BR False .
Second, if the
.B -d
command-line argument was specified, atoms of the form
.IR atom \(fm
are renamed to
.RI ! atom 
and atoms of the form
.IR atom \(fm(X)
are renamed to
.RI ! atom (X).
Note that, if the output is intended for use with sderes or lderes, this
second change affects semantics---it assumes that a pair of atoms named
.I atom
and
.IR atom \(fm
are intended to be the classical negations of each other.

.SH OPTIONS
.TP
.BR "\-u" ", " "\-\-uniquify"
Make killing atoms unique.  The default is to not do so.
.TP
.BR "\-U" ", " "\-\-no\-uniquify"
Do not make killing atoms unique.  This is the default.
.TP
.BR "\-d" ", " "\-\-rename-dual"
Rename atoms and predicates of the form
.IR a \(fm
to
.RI ! a .
.TP
.BR "\-D" ", " "\-\-no-rename-dual"
Do not perform such renamings.
.TP
.BR "\-k" ", " "\-\-killing-req"
Convert requirements into killing rules.  This is the default, and is required
for programs that will eventually be run under sderes or lderes.
.TP
.BR "\-K" ", " "\-\-no-killing-req"
Do not convert requirements.
.TP
.BR "\-t" ", " "\-\-time"
Perform timing.  The CPU time in seconds (as measured by
.BR clock (3))
is displayed to stderr after the simplification is performed.  This is the default.
.TP
.BR "\-n" ", " "\-\-no-time"
Do not display timing information.
.TP
.BR "\-V" ", " "\-\-verbose"
Display extra status information to stderr.  This flag causes
.B simplify
to print a message when entering each phase of computation.
.TP
.BR "\-m" ", " "\-\-messages"
Display only error messages (and possibly timing information, depending on the state of the
.B "\-t"
and
.B "\-n"
flags) to stderr.
.TP
.BR "\-q" ", " "\-\-quiet"
Do not display any messages (other than perhaps timing information) to 
stderr.
.TP
\fB\-o\fI filename\fR, \fB\-\-output=\fIfilename\fR
Send output to \fIfilename\fR.  The default is to send output to stdout.
.TP
.BR "\-v" ", " "\-\-version"
Print version information to stdout.  The information indicates the version numbers of
.BR simplify ", " liblp ", and " lpsms .
.TP
.BR "\-h" ", " "\-\-help"
Display usage information to stdout.

.SH "EXAMPLE USAGE"
\fClparse -1 prog1 | simplify -u -n -q -o prog1.out\fR

Takes the output of \(lq\fClparse -1 prog1\fR\(rq, simplifies it along
with making killing atoms unique, and puts the result in prog1.out.
Displays no messages or timing information to stderr.

\fCsimplify -V prog2.in\fR

Simplifies prog2.in without making killing atoms unique.  Displays the
transformed program to stdout, and the messages (with extra verbosity)
and timing information to stderr.

.SH AUTHOR
.RI "Neil Moore <" neil@cs.uky.edu ">"

.B simplify
is Copyright \(co 1999 the University of Kentucky.  See the file COPYRIGHT in the
.B lpsms
distribution for details.

.SH BUGS
.B simplify
does not currently support the
.B smodels 2
format.
.B lparse
can produce both
.B smodels 1
and
.B 2
format programs, and we do not deal with
.B smodels 2
special rule types, so support for the
.B smodels 2
format is not a high priority.

.SH SEE ALSO
.BR lparse.info ,
in the
.B lparse
source distribution.  This file describes the input format accepted by
.BR sm2dt .

.BR stratify "(1), " smdtconv "(1), " sm2dt "(1), " stable "(1), " sderes "(1), " lderes "(1)."
