.\" -*- nroff -*-
.TH lderes 1 "15 Nov 1999" "lpsms 0.5.1"
.\" Turn off hyphenation
.hlm 0
.\" Turn off adjustment
.na
.SH NAME
lderes - compute extensions of simple default theories
.SH SYNOPSIS
.B lderes
.RI "[" options "]"
.I file
.\" Back to standard adjustment
.ad
.\" Allow hyphenation, omit if you want no hyphenation in the man page
.hlm 1

.SH DESCRIPTION
The program
.B lderes
computes extensions of simple default theories.  There is a natural and rather simple embedding of
logic programs into simple default theories; in the case of a logic program, lderes computes stable
models of the program.  Input is in DeReS format, and consists of three files: a .dt file which
points to the other two files; a .dc file which contains the defaults, or rules, of the theory; and
a .thc file, containing the initial set of facts (literals or conjunctions of literals).  File
formats are described below.

.SH OPTIONS
.TP
.BR "\-1" ", " "\-\-single"
Compute a single extension.  This is the default.
.TP
.BR "\-a" ", " "\-\-all"
Compute all extensions.
.TP
\fB\-c\fI N\fR, \fB\-\-count=\fIN\fR
Compute the first
.I N
extensions.
.TP
.BR "\-t" ", " "\-\-time"
Perform timing.  The CPU time in seconds used by parsing and computation, as well as the CPU time
used in finding each extension (as measured by
.BR clock (3))
is displayed to stdout after the computation.  This is the default.
.TP
.BR "\-n" ", " "\-\-no-time"
Do not display timing information.
.TP
.BR "\-V" ", " "\-\-verbose"
Display extra information to stderr and stdout.  This flag causes
.B lderes
to display candidate numbers of extensions to stdout, and a number of status messages to stderr.
.TP
.BR "\-m" ", " "\-\-messages"
Display only error messages to stderr.  Display extensions (and possibly timing information,
depending on the state of the
.B "\-t"
and
.B "\-n"
flags) to stdout.
.TP
.BR "\-q" ", " "\-\-quiet"
Display nothing to stderr; display extensions and possibly timing information to stdout.
.TP
.BR "\-qq" ", " "\-\-very-quiet"
Display nothing to stderr.  Also do not display extensions to stdout; count them only.
Timing information may still be printed, depending on the state of the
.B "\-t"
and
.B "\-n"
flags.
.TP
.BR "\-v" ", " "\-\-version"
Print version information to stdout.  The information indicates the version numbers of
.B lderes 
and
.BR lpsms .
.TP
.BR "\-h" ", " "\-\-help"
Display usage information to stdout.

.SH "EXAMPLE USAGE"
\fClderes -t -q -c 3 prog1\fR

Compute the first three extensions of the program in prog1.dt,
prog1.thc, prog1.dc, and prog1.str.
.B lderes
will display the extensions and timing information, but no other status
messages, to stdout.

\fClderes -V -1 prog2\fR

Compute the first extension of the program found in prog2.dt,
prog2.thc, prog2.dc, and prog2.str.  Display that extension with its
candidate number and timing information to stdout, and other status
information to stderr.

.SH "FILE FORMATS"

When given a filename
.I file
on the command line, 
.B lderes
first opens
.IR file .dt.
This file should contain two lines (in addition to any comments):
.RS
\fCw = \fIwfile\fR
.br
\fCd = \fIdfile\fR
.RE
The default theory is then read from
.IR wfile .thc
and
.IR dfile .dc.
The first file contains expressions which are either literals or conjunctions of literals.
Expressions are separated by blank lines.  If
.B smdtconv
was used to produce the files, the
.B .thc
file will contain no expressions.  The second file contains a set of defaults of the
form:
.RS
\fC a : b1, b2, b3
.br
     -> g;
.RE
The prerequisite, a, is a literal, a conjunction of literals, or a disjunction of literals.
Conjunction is represented by \(lq&&\(rq, disjunction by \(lq||\(rq, and negation by \(lq!\(rq.  The
justifications b1 . . .  b3, are also literals, conjunctions of literals, or disjunctions of
literals.  Finally, the conclusion, g, is a literal or conjunction of literals.  Each default is
separated from the next by a blank line.  For the semantics of default logic, see [CMT].

In all three files, comments begin with \fC%\fR and extend to the end of a line.

.SH AUTHOR
.BR DeReS ,
a system for full default logic, was originally developed by Pawel Cholewinski.

Matt Levy
.RI "<" mattlevy@csr.uky.edu ">"
reimplemented
.B DeReS
as
.BR lderes ,
making great speed improvements in the process.
.B lderes
was made to conform to the command-line style of
.B sderes
and the rest of
.B lpsms
by Neil Moore
.RI "<" neil@cs.uky.edu ">."
Some fatal bugs were fixed by Artur Mikitiuk
.RI "<" artur@cs.uky.edu ">."

.B DeReS
and
.B lderes
are Copyright \(co 1992-1999 The University of Kentucky.  See the file COPYRIGHT in the
.B lpsms
distribution for details.

.SH BUGS
Maybe.

.SH "SEE ALSO"
.BR sm2dt "(1), " smdtconv "(1), " stable "(1), " sderes "(1), " smodels "(1)."
.br
[CMT] Pawel Cholewinski, Victor Marek, and Miroslaw Truszczynski,
\(lqDefault Reasoning System DeReS\(rq.  \fIProceedings of KR\fP, 1996.  pp
518\(en528.
