.\" -*- nroff -*-
.TH make-stable 1 "3 Mar 2000" "lpsms 0.5.1"
.\" Turn off hyphenation
.hlm 0
.\" Turn off adjustment
.na
.SH NAME
make-stable - process a logic program for use with stable
.SH SYNOPSIS
.B make-stable
.RI "[" options "]"
.RI "[" file "...]"
.\" Back to standard adjustment
.ad
.\" Allow hyphenation, omit if you want no hyphenation in the man page
.hlm 1

.SH DESCRIPTION
The program
.B make-stable
converts a logic program in
.B lparse
syntax to a logic program in
.B smodels 1
syntax, suitable for use with
.BR stable .
It also performs simplification and stratification.  It is a driver for
.BR lparse ", " simplify ", and " stratify .

.SH OPTIONS
.TP
.BR "\-t" ", " "\-\-time"
Perform timing.  The CPU time in seconds (as measured by
.BR clock (3))
is displayed to stderr after each phase of processing.  This is the default.
.TP
.BR "\-n" ", " "\-\-no-time"
Do not display timing information.
.TP
.BR "\-c" ", " "\-\-cleanup"
Delete all the intermediate files upon success of the processing.  This
is the default.
.TP
.BR "\-C" ", " "\-\-no-cleanup"
Do not delete intermediate files.
.TP
.BR "\-V" ", " "\-\-verbose"
Display extra status information to stderr.  This flag causes
.B make-stable
to print a message indicating the command line with which each component is run, as well as some
additional information.
.BR "\-VV" ", " "\-\-more-verbose"
Display extra status information to stderr.  This flag, in addition to providing all the status
messages that
.B "\-V"
does, also runs
.B simplify
and
.B stratify
with the
.B "\-V"
flag.
.TP
.BR "\-m" ", " "\-\-messages"
Display only error messages (and possibly timing information, depending on the state of the
.B "\-t"
and
.B "\-n"
flags) to stderr.
.TP
.BR "\-q" ", " "\-\-quiet"
Do not display any messages (other than perhaps timing information) to stderr.
.TP
\fB\-o\fI filename\fR, \fB\-\-output=\fIfilename\fR
Direct the output to the files \fIbase\fB.ord\fR and \fIbase\fB.str\fR.  If
.B "\-o"
is not specified, the base defaults to the name of the first input file specified on the command
line, or \(lqstable\(rq if none is specified.
.TP
\fB\-d\fI dir\fR, \fB\-\-directory=\fIdir\fR
Search for
.BR lparse ", " simplify ", and " stratify
in the directory
.IR dir .
If a program is not found in
.IR dir ,
it is searched for in the directory from which
.B make-stable
was run, then in the deafault search path.  Multiple
.B "\-d"
options are cumulative; the directories are searched in the order given.
.TP
\fB\-p\fI program\fR, \fB\-\-parser=\fIprogram\fR
Use
.I program
to ground programs and convert them to
.B smodels 1
format.  The default is \(lq\fClparse -1\fR\(rq.  Other possibilities are \(lq\fCparse\fR\(rq,
\(lq\fClparse -1 -d none\fR\(rq, and \(lq\fClparse -1 -d all\fR\(rq.  If the parser is specified
with
.BR -p ,
the
.B -d
flag is not used to find it.
.TP
.BR
.TP
.BR "\-v" ", " "\-\-version"
Print version information to stdout.  The information indicates the version numbers of
.BR make-stable ", " liblp ", " lpsms ", " simplify ", and " stratify .
.TP
.BR "\-h" ", " "\-\-help"
Display usage information to stdout.

.SH "EXAMPLE USAGE"
\fCmake-stable -VV -d/opt/lpsms/bin/ -o prog1 p1-1 p1-2\fR

Combine, simplify, and stratify p1-1 and p1-2 into prog1.ord with
stratification file prog1.str.  Search for
.BR lparse ", " simplify ", and " stratify
in /opt/lderes/bin/ before searching in the location of
.B make\-stable
and in $PATH.  Display extra status information to stderr, and have
.B simplify
and
.B stratify
do the same.

\fCmake-stable -q -n prog2 prog3 prog4\fR

Combine, simplify, and stratify prog2, prog3, and prog4 into prog2.ord
with stratification file prog1.str.  Display nothing to stderr or
stdout.

.SH COMPONENTS
Information on
.B simplify
and
.B stratify
can be found in the appropriate man pages (in section 1).  Information on
.B lparse
can be found in the
.B lparse
GNU info page, or in the \fClparse/\fR directory of the
.B lpsms
source distribution.

.SH AUTHOR
.RI "Neil Moore <" neil@cs.uky.edu ">"

.B make-stable
is Copyright \(co 1999 The University of Kentucky.  See the file COPYRIGHT in the
.B lpsms
distribution for details.

.SH BUGS
If one of the processes
.RB ( lparse ", " simplify ", or " stratify )
should die, temporary files (\fIfilename\fB.out\fR, \fIfilename\fB.sm\fR, or
\fIfilename\fB.sm.simp\fR) may be left over.  This is not necessarily a bug: these files can be
useful for determining exactly what caused the process to fail.

None of the programs (except
.BR lparse ,
of course) currently support the
.B smodels 2
format.
.B lparse
can produce both
.B smodels 1
and
.B 2
format programs, so support for the
.B smodels 2
format is not a high priority.

.SH SEE ALSO
.BR simplify "(1), " stratify "(1), " stable "(1)."

.BR lparse.info ,
in the
.B lparse
source distribution.  This file describes the input format accepted by
.BR make-stable ,
as well as providing general information over grounding and over logic programming with stable model
semantics.
