.\" -*- nroff -*-
.TH stable 1 "24 Feb 2000" "lpsms 0.5.1"
.\" Turn off hyphenation
.hlm 0
.\" Turn off adjustment
.na
.SH NAME
stable - compute stable models of logic programs
.SH SYNOPSIS
.B stable
.RI "[" options "]"
.RI "[" file "]"
.\" Back to standard adjustment
.ad
.\" Allow hyphenation, omit if you want no hyphenation in the man page
.hlm 1

.SH DESCRIPTION
The program
.B stable
computes stable models of logic programs.  It accepts logic programs
in smodels format, usually generated by
.B make-stable
or
.B lparse .

.SH OPTIONS
.TP
\fB\-s\fI strfile\fR, \fB\-\-stratification=\fIstrfile\fR
Read the stratification from
.IR strfile .
Stratification is in
.B DeReS
stratification format, as produced by
.BR stratify .
.TP
.BR "\-F" ", " "\-\-full-lookahead"
Perform full lookahead.  This option causes
.B stable
to spend more time in lookahead, in the hopes of reducing the total number of choice points.  This
option is the default.
.TP
.BR
.BR "\-S" ", " "\-\-sloppy-lookahead"
Do not perform full lookahead.  This option will most likely be removed soon.
.TP
.BR "\-1" ", " "\-\-single"
Compute a single stable model.  This is the default.
.TP
.BR "\-a" ", " "\-\-all"
Compute all stable models.
.TP
\fB\-c\fI N\fR, \fB\-\-count=\fIN\fR
Compute the first
.I N
stable models.
.TP
.BR "\-t" ", " "\-\-time"
Perform timing.  The CPU time in seconds used by parsing and computation, as well as the CPU time
used in finding each model (as measured by
.BR clock (3))
is displayed to stdout after the computation.  This is the default.
.TP
.BR "\-n" ", " "\-\-no-time"
Do not display timing information.
.TP
.BR "\-V" ", " "\-\-verbose"
Display extra information to stderr.  This flag causes
.B stable
to print a message to stderr upon entering each phase of processing.
.TP
.BR "\-m" ", " "\-\-messages"
Display only error messages to stderr.  Display stable models (and possibly timing information,
depending on the state of the
.B "\-t"
and
.B "\-n"
flags) to stdout.
.TP
.BR "\-q" ", " "\-\-quiet"
Display nothing to stderr; display stable models and possibly timing information to stdout.
.TP
.BR "\-qq" ", " "\-\-very-quiet"
Display nothing to stderr.  Also do not display stable models to stdout; count them only.
Timing information may still be printed, depending on the state of the
.B "\-t"
and
.B "\-n"
flags.
.TP
.BR "\-v" ", " "\-\-version"
Print version information to stdout.  The information indicates the version numbers of
.BR lderes ", " liblp ", and " lpsms .
.TP
.BR "\-h" ", " "\-\-help"
Display usage information to stdout.

.SH "EXAMPLE USAGE"
\fCstable -t -q -c 9 -S prog1.org -s prog1.str\fR

Compute the first nine extensions of the program in prog1.ord, using
sloppy lookahead and the stratification in prog1.str; display the
extensions and timing information, but no other status messages, to
stdout.

\fCsderes -qq -1 prog2.ord\fR

Determine whether prog2 has at least one extension; display timing
information, but not the stable model or anything else.  Use full
lookahead without stratification.

.SH AUTHOR
.B stable
was written by Neil Moore
.RI "<" neil@cs.uky.edu ">,"
but is closely based on the algorithms used in
.BR smodels .
.B smodels
was written by Patrik Simons
.RI "<" Patrik.Simons@hut.fi ">."

.B stable
is Copyright \(co 1999 The University of Kentucky.  See the file COPYRIGHT in the
.B lpsms
distribution for details.

.SH BUGS
.B stable
measures forced atoms and conflicts differently than does
.BR smodels .

Sometimes (actually, quite often) it is better to not use
stratification; the backchaining in the
.B smodels
algorithm allows for the choice of an atom in a higher stratum to expand more atoms in lower
strata.  Stratification simply forces
.B stable
to expand on some atom in the lower stratum, even when a higher stratum would have given better
results.  In general, this results in many more choice points than are actually necessary.

Additionally,
.B stable
keeps the original stratification throughout the computation.  It would be best to recompute the
stratification periodically based on the simplified program.  For example, if the atom
.B p
has been forced or chosen true, the stratum in which appears as a rule head may be split up, as
edges involving
.B p
may be ignored in the stratification graph.

.B stable
should support the
.B smodels
compute statement; then
.B simplify
would not be required for programs with global constraints (headless rules).

Finally,
.B stable
is in many cases much slower than
.BR smodels .
Most of this is probably due to implementation details, and can be
easily fixed.

.SH SEE ALSO
.BR lderes "(1), " sderes "(1), " stratify "(1), " simplify "(1), " make-stable "(1)."
.br
.B smodels
is available from
.BR http://saturn.tcs.hut.fi/pub/smodels/ .
A discussion of the algoritm may be found in:
.br
P. Simons.  `Towards constraint satisfaction through logic programs and the stable model semantics'.
Research Report 47, Helsinki University of Technology, Helsinki, Finland, August 1997.
.br
This report is available at
.BR http://www.tcs.hut.fi/Publications/reports/A47.ps.gz .
